# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pycograph', 'pycograph.helpers', 'pycograph.schemas']

package_data = \
{'': ['*']}

install_requires = \
['pydantic>=1.8,<2.0', 'redisgraph>=2.3,<3.0', 'typer==0.3.2']

entry_points = \
{'console_scripts': ['pycograph = pycograph.cli:app']}

setup_kwargs = {
    'name': 'pycograph',
    'version': '0.2.2',
    'description': '',
    'long_description': '# Pycograph\n\n![Pycograph](resources/pycograph_logo.png)\n\n**Explore your Python code with graph queries**\n\nPycograph creates a [RedisGraph](https://oss.redislabs.com/redisgraph/) model of your Python code. You can: \n\n* query it with [Cypher](https://oss.redislabs.com/redisgraph/commands/)\n* visualize it with [RedisInsight](https://redislabs.com/redis-enterprise/redis-insight/)\n\n![sample Redis Insight result](resources/sample_redis_insight.png)\n\n## Getting Started\n\nInstall Pycograph from PyPI:\n\n```\npip install pycograph\n```\n\nStart a Redis instance with the RedisGraph module and RedisInsight. E.g. via Docker containers:\n\n```\ndocker run -d -p 6379:6379 redislabs/redismod\ndocker run -d -v redisinsight:/db -p 8001:8001 redislabs/redisinsight:latest\n```\n\nVisit your RedisInstance at http://localhost:8001 in a browser.  \nConnect to your local Redis database.\n\nCreate a RedisGraph model of your project\'s code with the `pycograph load` command:\n\n```\npycograph load --project-dir ~/code/your-project --test-types\n```\n\nBy default, if you don\'t provide the `--project-dir` option, Pycograph tries to find Python code in the current working directory.  \n\n\nRun a query in RedisInsight. E.g.\n\n```\nGRAPH.QUERY "your-project" "MATCH (n) return n"\n```\n\n## Options\n\n* `--project-dir`: The root directory of the Python project you want to analyze. If you omit this option, Pycograph will search for `.py` files in your current working directory.\n* `--graph-name`: Specifies the name of the generated graph. Default: the name of the project directory.\n* `--overwrite`: If a graph with this name exists overwrite it. If you don\'t provide this flag, the new nodes and edges will be appended to the graph.\n* `--test-types`: Determine the types of tests based on the subdirectories of the `tests` directory.\n* `--redis-host`: The host of the Redis instance. Default: localhost\n* `--redis-port`: The port of the Redis instance. Default: 6379 \n* `--version`: Print Pycograph version and exit.\n\n## Limitations\n\nPycograph is in beta version.\n\nIt creates a basic model with focus on the relationships betweeen the different parts of the code base. Even that model might be incomplete, ignoring some less common syntax. The goal is to provide some useful insight, not to create an exhaustive model.\n\nIf Pycograph finds a syntax error, it skips the module containing the syntax error and tries to build a model from the rest of the code base.\n\nBelow are some of the limitations. If you bump into other limitations, please open a GitHub issue.\n\n### Imports\n\nThe following imports will be ignored by Pycograph:\n\n* imported external packages\n* `import *` syntax\n* variables\n* globals\n\n### Calls\n\n* All the limitations of the imports.\n* Resolving longer calls of more than 2 levels.\n\n### Other Known Limitations\n\n* No support for `.py` files containing Jinja templates (e.g. cookiecutter)\n\n## How Does It Work?\n\n![flow of loading code: source => RedisGraph](resources/pycograph_architecture.png)\n\n### Libraries used:\n\n* [ast](https://docs.python.org/3/library/ast.html) module of the Python standard library for the abstract syntax tree\n* [Pydantic](https://pydantic-docs.helpmanual.io) both for the models of the intermediate objects and for the settings\n* [redisgraph-py](https://github.com/RedisGraph/redisgraph-py) for creating the RedisGraph model\n* [typer](https://typer.tiangolo.com/) for the command line interface\n\n## Contributing\n\nsee the [Contributing guide](CONTRIBUTING.md)',
    'author': 'reka',
    'author_email': 'reka@hey.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pycograph.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
